library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.common.all;

package addrgenPckg is

component addrgen
generic(
	w: 					natural := 512;	-- scanline width in pixels
	h: 					natural := 512;	-- scanline height in pixels
	datawidth:	natural	:= 8;		-- RAM bit width
	pixelWidth: natural := 4		-- number of bits per pixel
);
port(
	pixelCnt:	in unsigned(10 downto 0);		-- index of current pixel in scanline
	lineCnt:	in unsigned(10 downto 0);		-- index of current scanline in image frame
	offset:		in unsigned(31 downto 0);		-- RAM address offset
	addr:			out unsigned(31 downto 0);	-- RAM address of current pixel
	ce_n:			out std_logic;							-- RAM chip-enable
	oe_n:			out std_logic;							-- RAM output-enable
	pixelRdy:	out std_logic;							-- pixel data from RAM is available
	blank:		out std_logic								-- image is blanked
);
end component;

end package;




library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.common.all;



-- Compute an address for the pixel data based on the index of the
-- pixel within the scanline and the scanline index within the image frame.

entity addrgen is
generic(
	w: 					natural := 512;	-- scanline width in pixels
	h: 					natural := 512;	-- scanline height in pixels
	datawidth:	natural	:= 8;		-- RAM bit width
	pixelWidth: natural := 4		-- number of bits per pixel
);
port(
	pixelCnt:	in unsigned(10 downto 0);		-- index of current pixel in scanline
	lineCnt:	in unsigned(10 downto 0);		-- index of current scanline in image frame
	offset:		in unsigned(31 downto 0);		-- RAM address offset
	addr:			out unsigned(31 downto 0);	-- RAM address of current pixel
	ce_n:			out std_logic;							-- RAM chip-enable
	oe_n:			out std_logic;							-- RAM output-enable
	pixelRdy:	out std_logic;							-- pixel data from RAM is available
	blank:		out std_logic								-- image is blanked
);
end addrgen;



architecture arch of addrgen is
constant pixPerByte:	natural := datawidth/pixelWidth;	-- number of pixels packed into each word of RAM
-- number of bits needed to represent the index of the pixel in the data word and
-- the width and height of the image
constant fieldLen:	natural := log2(pixPerByte);
constant widthLen:	natural := log2(w);
constant heightLen:	natural := log2(h);
-- number of RAM address bits required to fetch pixel data bytes
constant addrLen:		natural := heightLen + widthLen - fieldLen;
signal field:			natural;		-- field of pixel within data byte
signal ramEnable:	std_logic;
signal addr_i:		unsigned(addr'range);	-- internal copy of address
begin
	-- concatenate scanline and pixel indices to get RAM address of pixel
	addr_i(addrLen-1 downto 0) <= lineCnt(heightLen-1 downto 0) & pixelCnt(widthLen-1 downto fieldLen);
	addr_i(addr_i'high downto addrLen) <= TO_UNSIGNED(0,addr_i'length-addrLen);	-- zero the upper address bits
	
	-- add address offset
	addr <= addr_i + offset;

	-- get the index of the pixel data field with a single byte of RAM data
	field <= 0 when fieldLen=0 else TO_INTEGER(pixelCnt(max(fieldLen-1,0) downto 0));
	
	-- enable the RAM when within the image frame and fetching the first byte of data
	ramEnable <= YES when TO_INTEGER(pixelCnt) < w and TO_INTEGER(lineCnt) < h and field=0 else NO;
	
	ce_n <= LO;																-- always enable the RAM
	oe_n <= LO when ramEnable = YES else HI;	-- enable RAM outputs when fetching pixel data
	
	pixelRdy <= ramEnable;	-- indicate pixel data is available when RAM is enabled
	
	-- blank the video when outside the width or height of the image frame
	blank <= NO when TO_INTEGER(pixelCnt) < w and TO_INTEGER(lineCnt) < h else YES;
end arch;
