library IEEE,unisim;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use unisim.vcomponents.all;
use WORK.common.all;


package busdev is

component busdev
	generic(
		FREQ:	natural := 50_000;			-- operating frequency in KHz
		HDATA_WIDTH: natural := 16;		-- host-side data width
		DDATA_WIDTH: natural := 16;		-- device-side data width
		HADDR_WIDTH: natural := 20;		-- host-side address width
		DADDR_WIDTH: natural := 20;		-- device-side address width
		TRD: natural := 100;					-- device read access time (ns)
		TWR: natural := 100						-- device write access time (ns)
	);
	port(
		clk:	in	std_logic;					-- master clock
		cke:	in	std_logic;					-- master clock enable

		-- host side
		hRst:		in	std_logic;				-- reset
		hRd:		in	std_logic;			-- read data
		hWr:		in	std_logic;			-- write data
		hAddr:	in	unsigned(HADDR_WIDTH-1 downto 0);		-- address from host
		hDIn:		in	unsigned(HDATA_WIDTH-1 downto 0);		-- data from host
		hDOut:	out	unsigned(HDATA_WIDTH-1 downto 0);		-- data to host
		busy:		out	std_logic;				-- read/write in progress
		done:		out	std_logic;				-- read/write done
		cntlState:	out std_logic_vector(3 downto 0);		-- controller state

		-- device side
		dCe_n:	out	std_logic;				-- enable device
		dWe_n:	out	std_logic;				-- enable write to device
		dOe_n:	out	std_logic;				-- enable device outputs
		dAddr:	out	unsigned(DADDR_WIDTH-1 downto 0);		-- address
		dDIn:		in	unsigned(DDATA_WIDTH-1 downto 0);		-- input databus
		dDOut:	out unsigned(DDATA_WIDTH-1 downto 0)		-- output databus
	);
end component;

end package busdev;



library IEEE,unisim;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use unisim.vcomponents.all;
use WORK.common.all;

entity busdev is
	generic(
		FREQ:	natural := 50_000;			-- operating frequency in KHz
		HDATA_WIDTH: natural := 16;		-- host-side data width
		DDATA_WIDTH: natural := 16;		-- device-side data width
		HADDR_WIDTH: natural := 20;		-- host-side address width
		DADDR_WIDTH: natural := 20;		-- device-side address width
		TRD: natural := 100;					-- device read access time (ns)
		TWR: natural := 100						-- device write access time (ns)
	);
	port(
		clk:	in	std_logic;					-- master clock
		cke:	in	std_logic;					-- master clock enable

		-- host side
		hRst:		in	std_logic;				-- reset
		hRd:		in	std_logic;			-- read data
		hWr:		in	std_logic;			-- write data
		hAddr:	in	unsigned(HADDR_WIDTH-1 downto 0);		-- address from host
		hDIn:		in	unsigned(HDATA_WIDTH-1 downto 0);		-- data from host
		hDOut:	out	unsigned(HDATA_WIDTH-1 downto 0);		-- data to host
		busy:		out	std_logic;				-- read/write in progress
		done:		out	std_logic;				-- read/write done
		cntlState:	out std_logic_vector(3 downto 0);		-- controller state

		-- device side
		dCe_n:	out	std_logic;				-- enable device
		dWe_n:	out	std_logic;				-- enable write to device
		dOe_n:	out	std_logic;				-- enable device outputs
		dAddr:	out	unsigned(DADDR_WIDTH-1 downto 0);		-- address
		dDIn:		in	unsigned(DDATA_WIDTH-1 downto 0);		-- input databus
		dDOut:	out unsigned(DDATA_WIDTH-1 downto 0)		-- output databus
	);
end busdev;



architecture arch of busdev is

	constant RD_CYCLES:		natural := 1 + ((TRD * FREQ) / NORM);	-- read access time
	constant WR_CYCLES:		natural := 1 + ((TWR * FREQ) / NORM);	-- write access time
	
	-- states of the device controller state machine
	type cntlState is (
		INIT,					-- initialization - waiting for power-on initialization to complete
		RW,						-- wait for read/write operations to device
		RDDONE,				-- indicate that the device read is done
		WRDONE				-- indicate that the device write is done
	);
	signal state_r, state_next: cntlState;	-- state register and next state
	
	-- registers
	signal wrFlag_r, wrFlag_next: std_logic;					-- 1 when writing data to device
	signal rdFlag_r, rdFlag_next: std_logic;					-- 1 when reading data from device

	-- register that counts down times for read/write perations
	signal timer_r, timer_next: unsigned(log2(RD_CYCLE+WR_CYCLE+1)-1 downto 0);	-- current SDRAM op time

begin

	dAddr <= hAddr(dAddr'range);	-- connect host address bus to device address bus
	hDOut	<= dDIn(hDOut'range);		-- connect device data bus to host data bus
	dDOut <= hDIn(dDOut'range);		-- connect host data bus to device data bus

	combinatorial: process()
	begin
		-- default operations
		busy	<= NO;			-- no pending device operation
		done 	<= NO;			-- pending device operation is not done
		dCe_n	<= '1';			-- disable device
		dWe_n	<= '1';			-- disable write to device
		dOe_n <= '1';			-- disable device outputs
	
		-- default register updates
		state_next	<= state_r;
		rdFlag_next	<= rdFlag_r;
		wrFlag_next	<= wrFlag_r;
	
		-- update timer
		if timer_r /= TO_UNSIGNED(0,timer_r'length) then
			timer_next <= timer_r - 1;
		else
			timer_next <= timer_r;
		end if;
		
		-- ***** compute next state and outputs *****
		
		-- don't do anything if the previous operation has not completed yet.
		-- Place this before anything else so operations in the previous state
		-- complete before any operations in the new state are executed.
		if timer_r /= TO_UNSIGNED(0,timer_r'length) then
			if rdFlag = '1' then
				dCe_n <= '0';
				dOe_n <= '0';
				busy <= YES;
				done <= NO;
			elsif wrFlag = '1' then
				dCe_n <= '0';
				dWe_n <= '0';
				busy <= YES;
				done <= NO;
			end if;
			cntlState <= "0000";

		elsif state_r = INIT then
			-- initiate wait for device power-on initialization
			busy <= YES;
			done <= NO;
			state_next <= RW;
			cntlState <= "0001";

		-- do nothing but wait for read or write operations
		elsif state_r = RW then
			if rd = YES then
				-- the host has initiated a read operation
				rdFlag_next <= YES;		-- set flag to indicate a read operation is in progress
				dCe_n <= '0';
				dOe_n <= '0';
				busy <= YES;
				done <= NO;
				timer_next <= TO_UNSIGNED(RD_CYCLE,timer_next'length);	-- setup timer for read access
				state_next <= RDDONE;	-- read the data from device after the access time
				cntlState <= "0010";
			elsif wr = YES then
				-- the host has initiated a write operation
				wrFlag_next <= YES;		-- set flag to indicate a write operation is in progress
				dCe_n <= '0';
				dWe_n <= '0';
				busy <= YES;
				done <= NO;
				timer_next <= TO_UNSIGNED(WR_CYCLES,timer_next'length);	-- setup timer for write access
				state_next <= WRDONE;	-- write the data to the device after the access time
				cntlState <= "0011";
			else
				busy <= NO;
				done <= NO;
				cntlState <= "0100";
			end if;

		-- enter this state when the data read from the SDRAM is available
		elsif state_r = RDDONE then
			rdFlag_next <= NO;	-- set flag to indicate the read operation is over
			dCe_n <= '0';
			dOe_n <= '0';
			busy <= YES;
			done <= YES;
			state_next <= RW;		-- go back and do another read/write operation
			cntlState <= "0101";

		-- enter this state when the data is written to the SDRAM
		elsif state_r = WRDONE then
			wrFlag_next <= NO;	-- set flag to indicate the write operation is over
			dCe_n <= '1';
			dWe_n <= '1';
			busy <= YES;
			done <= YES;
			state_next <= RW;		-- go back and do another read/write operation
			cntlState <= "0110";

		-- no operation
		else
			cntlState <= "0111";
		
		end if;
						
	end process combinatorial;


	-- update registers on the rising clock edge	
	update: process(clk)
	begin
		if clk'event and clk='1' then
			if rst = YES then
				state_r		<= INIT;
				rdFlag_r	<= NO;
				wrFlag_r	<= NO;
				timer_r		<= TO_UNSIGNED(0,timer_r'length);
			elsif cke = YES then
				state_r		<= state_next;
				rdFlag_r	<= rdFlag_next;
				wrFlag_r	<= wrFlag_next;
				timer_r		<= timer_next;
			else
				state_r		<= state_r;
				rdFlag_r	<= rdFlag_r;
				wrFlag_r	<= wrFlag_r;
				timer_r		<= timer_r;
			end if;
		end if;
	end process update;

end arch;
