library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.common.all;

package colormapPckg is

component colormap
generic(
	datawidth:	natural;		-- width of RAM
	pixelWidth: natural			-- number of bits per pixel
);
port(
	rst:		in std_logic;
	clk:		in std_logic;
	blank:	in std_logic;		-- image blanking signal
	load:		in std_logic;		-- image data byte is available
	pixdata: in unsigned(datawidth-1 downto 0);	-- image data byte containing pixels
	rgb:		out unsigned(5 downto 0)	-- RGB for the current pixel
);
end component;

end package;



library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.common.all;



-- This module loads a data byte when its availability from RAM
-- is signaled.  Then it shifts this data byte on successive clocks
-- and generates the RGB values for each pixel field.

entity colormap is
generic(
	datawidth:	natural;		-- width of RAM
	pixelWidth: natural			-- number of bits per pixel
);
port(
	rst:		in std_logic;
	clk:		in std_logic;
	blank:	in std_logic;		-- image blanking signal
	load:		in std_logic;		-- image data byte is available
	pixdata: in unsigned(datawidth-1 downto 0);	-- image data byte containing pixels
	rgb:		out unsigned(5 downto 0)	-- RGB for the current pixel
);
end colormap;



architecture arch of colormap is
signal pixelReg, pixelReg_next: unsigned(pixdata'range); -- pixel shift register
signal pixel: natural;	-- pixel value
begin

-- load the pixel data when its availability is signaled and then shift it
-- right by the pixel width on each following clock
pixelReg_next <= pixdata when load=YES or pixelWidth=datawidth else
									pixdata(pixelWidth-1 downto 0) & pixelReg(pixelReg'length-1 downto min(pixelWidth,pixelReg'length-1));

-- update the pixel data shift register
process(clk)
begin
	if clk'event and clk=HI then
		if rst=HI then
			pixelReg <= TO_UNSIGNED(0,pixelReg'length);	-- clear shift reg on reset
		else
			pixelReg <= pixelReg_next;
		end if;
	end if;
end process;

-- extract the current pixel value from the lower bits of the shift register
pixel <= TO_INTEGER(pixelReg(pixelWidth-1 downto 0));

-- map the pixel value into RGB values
process(blank,pixel)
begin
	if blank = YES then
		rgb <= "000000";	-- blank RGB to black
	else
		case pixelWidth is
			when 1 =>	-- 1-bit pixels, two possible colors
				case pixel is
					when 0 =>      rgb<="000000";	-- black
					when others => rgb<="111111";	-- white
				end case;
			when 2 =>	-- 2-bit pixels, 4 possible colors
				case pixel is
					when 0 =>      rgb<="000000";	-- black
					when 1 =>      rgb<="110000";	-- red
					when 2 =>      rgb<="001100";	-- green
					when others => rgb<="000011";	-- blue
				end case;
			when 4 =>	-- 4-bit pixels, 16 possible colors
				case pixel is
					when 0 =>      rgb<="000000";	-- black
					when 1 =>      rgb<="101010";	-- light-gray
					when 2 =>      rgb<="111111";	-- white
					when 3 =>      rgb<="110000";	-- red
					when 4 =>      rgb<="001100";	-- green
					when 5 =>      rgb<="000011";	-- blue
					when 6 =>      rgb<="111100";	-- yellow
					when 7 =>      rgb<="110011";	-- magenta
					when 8 =>      rgb<="001111";	-- cyan
					when 9 =>      rgb<="100000";	-- dark-red
					when 10 =>     rgb<="001000";	-- dark-green
					when 11 =>     rgb<="000010";	-- dark-blue
					when 12 =>     rgb<="101000";	-- tan
					when 13 =>     rgb<="100010";	-- purple
					when 14 =>     rgb<="001010";	-- teal
					when 15 =>     rgb<="010101";	-- dark-gray
					when others => rgb<="000000";	-- black
				end case;
			when 8 =>	-- 8-bit pixels, just use lower 6-bits to drive RGB
					rgb <= TO_UNSIGNED(pixel,rgb'length);
			when others =>
					rgb <= TO_UNSIGNED(pixel,rgb'length);
		end case;
	end if;
end process;

end arch;
