library ieee,work;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.vgaPckg.all;
use work.common.all;


--
-- simple circuit to grab frames of video data
--
entity fg is
port(
	fpga_clk:			in		std_logic_vector(1 downto 0);	-- main clock
	vidin_clk:			in		std_logic;							-- SAA7114 video clock (27 MHz)
	vidin_idq:			in		std_logic;							-- SAA7114 data qualifier
	vidin_igph:			in		std_logic;							-- SAA7114 horiz. sync
	vidin_igpv:			in		std_logic;							-- SAA7114 vert. sync
	vidin_ipd:			in		unsigned(7 downto 0);			-- SAA7114 data
	vidin_itri:			out	std_logic;							-- SAA7114 tristate-enable
	vidin_itrdy:		out	std_logic;							-- SAA7114 target-ready
	vidout_clk:			out	std_logic;							-- THS8133B clock
	vidout_hsync_n:	out	std_logic;							-- THS8133B horiz. sync
	vidout_vsync_n:	out	std_logic;							-- THS8133B vert. sync
	vidout_blank_n:	out	std_logic;							-- THS8133B blanking
	vidout_gy:			out	unsigned(9 downto 0);			-- THS8133B green component
	vidout_rcr:			out	unsigned(9 downto 0);			-- THS8133B red component
	vidout_bcb:			out	unsigned(9 downto 0);			-- THS8133B blue component
	ram_ce_n:			out	std_logic;							-- SRAM chip-enable
	pb_oe_n:				out	std_logic;							-- SRAM output-enable
	pb_we_n:				out	std_logic;							-- SRAM write-enable
	pb_ub_n:				out	std_logic;							-- SRAM upper-byte enable
	pb_lb_n:				out	std_logic;							-- SRAM lower-byte enable
	pb_ecs0_n:			out std_logic;
	pb_a:					out	unsigned(17 downto 0);			-- SRAM address bus
	pb_d:					inout	unsigned(15 downto 0)			-- SRAM data bus
);
end fg;



architecture arch of fg is

-- states for the frame grabber	
type video_state is (
	wait_for_vert_sync,		-- waiting for low-level of vert. sync
	wait_for_vert_sync_end,	-- waiting for rising-edge of vert. sync
	wait_for_horiz_sync,		-- waiting for low-level of horiz. sync
	wait_for_horiz_sync_end,	-- waiting for rising edge of horiz. sync
	active_video_chroma_b,	-- getting Cb chroma value
	active_video_lum_b,		-- getting Y luminance value
	active_video_chroma_r,	-- getting Cr chroma value
	active_video_lum_r,		-- getting Y luminance value
	display						-- displaying video frame
);

signal vid_r, vid_next:		video_state;
signal clk:				std_logic;
signal reset:			std_logic;
signal reset_cnt:		unsigned(2 downto 0);
signal grab_cntrlo:	unsigned(9 downto 0);
signal grab_cntrhi:	unsigned(17 downto 10);
signal grab_addr:		unsigned(17 downto 0);
signal disp_addr:		unsigned(17 downto 0);
signal grab:			std_logic;
signal display_frame:	std_logic;
signal luminance:		unsigned(7 downto 0);
signal status:			unsigned(7 downto 0);
signal refresh:		std_logic;
signal refresh_cnt:	unsigned(24 downto 0);

begin

	clk				<= vidin_clk;	-- rename master clock
	vidin_itri		<= HI;			-- enable the SAA7114 I-port
	vidin_itrdy		<= HI;			-- tell the SAA714 we are ready
--	vidout_blank_n	<= HI;			-- disable the blanking of the THS8133B

	-- generate a reset
	process(clk)
	begin
		if(clk'event and clk=HI) then
			if(reset_cnt="111") then
				reset <= LO;
			else
				reset <= HI;
				reset_cnt <= reset_cnt + 1;
			end if;
		end if;
	end process;

	-- generate a display refresh
	process(clk)
	begin
		if(clk'event and clk=HI) then
			refresh_cnt <= refresh_cnt + 1;
			if(refresh_cnt=TO_UNSIGNED(0,refresh_cnt'length)) then
				refresh <= HI;
			else
				refresh <= LO;
			end if;
		end if;
	end process;

	pb_ecs0_n <= reset;

	-- simple state machine that interprets the data stream from the SAA7114
	-- and latches the luminance component of the video.  The current state
	-- of the machine is indicated on the upper 8 bits of the RAM data bus
	-- while the luminance data is output on the lower 8 bits.
	process(clk,reset)
	begin

		if(reset=HI) then
			vid_next <= wait_for_vert_sync;
			status <= "00000000";		-- indicate reset state

		-- drive state machine with video clock from SAA7114
		elsif(clk'event and clk=HI) then

			if(vidin_idq = HI) then
   
   			grab <= NO;
   			display_frame <= NO;
   
   			case vid_next is
   
   			when wait_for_vert_sync =>
   				status <= "00000001";
   				if(vidin_igpv=LO) then
   					vid_next <= wait_for_vert_sync_end;
   				end if;
   
   			when wait_for_vert_sync_end =>
   				status <= "00000010";
   				grab_cntrlo <= (others=>'0');
   				grab_cntrhi <= (others=>'0');
   				if(vidin_igpv=HI) then
   					vid_next <= wait_for_horiz_sync;
   				end if;
   
   			when wait_for_horiz_sync =>
   				status <= "00000100";
   				if(vidin_igph=LO) then
   					vid_next <= wait_for_horiz_sync_end;
   				end if;
   
   			when wait_for_horiz_sync_end =>
   				status <= "00001000";
   				grab_addr <= grab_cntrhi & grab_cntrlo;
   				if(vidin_igph=HI) then
   					grab_cntrlo <= (others=>'0');
   					grab_cntrhi <= grab_cntrhi + 1;
   					vid_next <= active_video_lum_b;
   				end if;
   
   			when active_video_chroma_b =>	-- chroma B data from video stream
   				status <= "00010000";
   				if(vidin_igpv=LO) then
   					vid_next <= display;
   				elsif(vidin_igph=LO) then
   					vid_next <= wait_for_horiz_sync_end;
   				else
   					vid_next <= active_video_lum_b;
   				end if;
   
   			when active_video_lum_b =>	-- latch luminance data from video stream
   				status <= "00100000";
   				if(vidin_igpv=LO) then
   					vid_next <= display;
   				elsif(vidin_igph=LO) then
   					vid_next <= wait_for_horiz_sync_end;
   				else
   					vid_next <= active_video_chroma_r;
   					luminance <= vidin_ipd;
--   					luminance <= grab_cntrlo(7 downto 0);
   					grab <= YES;
   					grab_cntrlo <= grab_cntrlo + 1;
   					grab_addr <= grab_cntrhi & grab_cntrlo;
   				end if;
   
   			when active_video_chroma_r =>	-- chroma R data from video stream
   				status <= "01000000";
   				if(vidin_igpv=LO) then
   					vid_next <= display;
   				elsif(vidin_igph=LO) then
   					vid_next <= wait_for_horiz_sync_end;
   				else
   					vid_next <= active_video_lum_r;
   				end if;
   
   			when active_video_lum_r =>	-- latch luminance data from video stream
   				status <= "10000000";
   				if(vidin_igpv=LO) then
   					vid_next <= display;
   				elsif(vidin_igph=LO) then
   					vid_next <= wait_for_horiz_sync_end;
   				else
   					vid_next <= active_video_chroma_b;
   					luminance <= vidin_ipd;
--   					luminance <= grab_cntrlo(7 downto 0);
   					grab <= YES;
   					grab_cntrlo <= grab_cntrlo;
   					grab_addr <= grab_cntrhi & grab_cntrlo;
   				end if;
   
   			when display =>	-- display stored frame data
   				status <= "11110000";
   				display_frame <= YES;
   				if(refresh=HI) then
   					vid_next <= wait_for_vert_sync;
   				end if;
   
   			when others =>	-- handle erroneous states
   				status <= "11111111";
   				vid_next <= wait_for_vert_sync;

				end case;
			end if;

		end if;

	end process;

	ram_ce_n <= LO;
	pb_oe_n <= HI when (display_frame=NO) else LO;
	pb_we_n <= LO when (grab=YES) and (clk=HI) else HI;
	pb_ub_n <= HI;
	pb_lb_n <= LO;

	pb_d(7 downto 0) <= luminance when (display_frame=NO) else (others=>'Z');
	pb_d(15 downto 8) <= status;
	pb_a(17 downto 0) <= grab_addr when grab=YES else disp_addr;
	vidout_gy  <= pb_d(7 downto 0) & "00";
	vidout_rcr <= pb_d(7 downto 0) & "00";
	vidout_bcb <= pb_d(7 downto 0) & "00";
--	vidout_gy  <= pb_d(7 downto 0) & "00";
--	vidout_rcr <= "0" & pb_d(7 downto 0) & "0";
--	vidout_bcb <= "00" & pb_d(7 downto 0);

	-- VGA generator gets data from the SRAM and sends it to the
	-- THS8133B where it is displayed on a VGA monitor
	u1: vga
		generic map(
			freq => 27_000,
			datawidth => 8,
			pixbits => 8,
			w => 720,
			h => 240
			)
		port map(
			rst => reset,
			clk => clk,
			data => TO_UNSIGNED(0,8),
			addrOffset => TO_UNSIGNED(0,32),
			address(17 downto 0) => disp_addr(17 downto 0),
			address(31 downto 18) => open,
			ce_n => open,
			oe_n => open,
			we_n => open,
			vsync_n => vidout_vsync_n,
			hsync_n => vidout_hsync_n,
			blank_n => vidout_blank_n,
			rgb => open
		);

	vidout_clk <= clk;

end arch;

