library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.common.all;

package syncPckg is

component sync
generic(
	freq: 	natural	:= 50_000;	-- master clock frequency (in KHz)
	period:	natural := 32;			-- period of sync pulse (in us)
	start:	natural := 26;			-- time sync pulse starts within the period (in us)
	width:	natural := 4				-- width of sync pulse (in us)
);
port(
	rst: 		in std_logic;				-- reset
	clk: 		in std_logic;				-- master clock
	cke: 		in std_logic;				-- clock-enable
	sync: 	out std_logic;			-- sync pulse
	gate: 	out std_logic;			-- single-clock pulse at end of sync pulse
	cnt: 		out unsigned(10 downto 0)	-- output the timing counter value
);
end component;

end package;




library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use work.common.all;



-- Generate an active-low sync pulse within a waveform period.
-- Also output the value of the counter used for timing so that
-- it can be used in generating an address for a video RAM.

entity sync is
generic(
	freq: 	natural	:= 50_000;	-- master clock frequency (in KHz)
	period:	natural := 32;			-- period of sync pulse (in us)
	start:	natural := 26;			-- time sync pulse starts within the period (in us)
	width:	natural := 4				-- width of sync pulse (in us)
);
port(
	rst: 		in std_logic;				-- reset
	clk: 		in std_logic;				-- master clock
	cke: 		in std_logic;				-- clock-enable
	sync: 	out std_logic;			-- sync pulse
	gate: 	out std_logic;			-- single-clock pulse at end of sync pulse
	cnt: 		out unsigned(10 downto 0)	-- output the timing counter value
);
end sync;



architecture arch of sync is
constant	norm: 			natural := 1000;	-- normalization factor for us * KHz
constant	cycPeriod:	natural := (period * freq)/norm;	-- sync wave period in clock cycles
constant	cycStart:		natural := (start * freq)/norm;		-- sync pulse start in cycles
constant	cycWidth:		natural := (width * freq)/norm;		-- sync pulse width in cycles
constant	cycEnd:			natural := cycStart + cycWidth;		-- sync pulse end in cycles
signal		cntr, cntr_next: natural;			-- counter for timing sync pulse waveform
begin

-- increment counter and wrap around to zero at end of period
cntr_next <= 0 when cntr = cycPeriod-1 else cntr+1;

-- generate active-low sync pulse within waveform period
sync <= LO when cntr >= cycStart and cntr < cycEnd else HI;

-- generate gate signal at end of sync pulse
gate <= YES when cntr = cycEnd else NO;

-- output counter value
cnt <= TO_UNSIGNED(cntr,cnt'length);

-- update counter
process(clk)
begin
	if clk'event and clk=HI then
		if cke=YES then
			if rst=YES then
				cntr <= 0;
			else
				cntr <= cntr_next;
			end if;
		end if;
	end if;
end process;

end arch;
