library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_Std.all;
use work.syncPckg.all;
use work.addrgenPckg.all;
use work.colormapPckg.all;
use work.common.all;


package vgaPckg is

component vga
generic(
	freq: 		natural := 50_000;	-- master clock frequency (KHz)
	datawidth: natural:= 8;				-- width of RAM
	pixBits:	natural := 4;				-- number of bits per pixel (1,2,4,8)
	w:				natural := 256;			-- width of image in pixels
	h:				natural := 128			-- height of image in scanlines
);
port(
	rst:			in std_logic;								-- synchronous reset
	clk:			in std_logic;								-- master dot-clock
	data:			in unsigned(datawidth downto 0);		-- pixel-data from RAM
	addrOffset:	in unsigned(31 downto 0);	-- address offset to RAM 
	address:	out unsigned(31 downto 0);	-- pixel-data address to RAM 
	ce_n:			out std_logic;							-- RAM chip-enable
	oe_n:			out std_logic;							-- RAM output-enable
	we_n:			out std_logic;							-- RAM write-enable
	vsync_n:	out std_logic;							-- VGA vertical sync
	hsync_n:	out std_logic;							-- VGA horizontal sync
	blank_n:	out std_logic;							-- blanking signal
	rgb:			out unsigned(5 downto 0)		-- VGA red-green-blue signals
);
end component;

end package;



library IEEE,work;
use IEEE.std_logic_1164.all;
use IEEE.numeric_Std.all;
use work.syncPckg.all;
use work.addrgenPckg.all;
use work.colormapPckg.all;
use work.common.all;



entity vga is
generic(
	freq: 		natural := 50_000;	-- master clock frequency (KHz)
	datawidth: natural:= 8;				-- width of RAM
	pixBits:	natural := 4;				-- number of bits per pixel (1,2,4,8)
	w:				natural := 256;			-- width of image in pixels
	h:				natural := 128			-- height of image in scanlines
);
port(
	rst:			in std_logic;								-- synchronous reset
	clk:			in std_logic;								-- master dot-clock
	data:			in unsigned(datawidth downto 0);		-- pixel-data from RAM
	addrOffset:	in unsigned(31 downto 0);	-- address offset to RAM 
	address:	out unsigned(31 downto 0);	-- pixel-data address to RAM 
	ce_n:			out std_logic;							-- RAM chip-enable
	oe_n:			out std_logic;							-- RAM output-enable
	we_n:			out std_logic;							-- RAM write-enable
	vsync_n:	out std_logic;							-- VGA vertical sync
	hsync_n:	out std_logic;							-- VGA horizontal sync
	blank_n:	out std_logic;							-- blanking signal
	rgb:			out unsigned(5 downto 0)		-- VGA red-green-blue signals
);
end entity;



architecture arch of vga is
signal vcc:				std_logic;	-- logic hi
signal vertGate:	std_logic;	-- clock-enable for vertical sync generator
signal hsync_i:		std_logic;	-- internal copy of horiz. sync
signal hsync_dly:	std_logic_vector(3 downto 1);	-- 3 cycle delays for hsync
signal vsync_i:		std_logic;	-- internal copy of vert. sync
signal vsync_dly:	std_logic_vector(3 downto 1);	-- 3 cycle delays for vsync
signal pixelCnt:	unsigned(10 downto 0);	-- index of current pixel in scanline
signal lineCnt:		unsigned(10 downto 0);	-- index of current scanline in frame
signal addr_i:		unsigned(31 downto 0);	-- internal copy of RAM address
signal addr_dly:	unsigned(31 downto 0);	-- 1 cycle delay for address
signal ce_i:			std_logic;	-- internal copy of RAM chip-enable
signal ce_dly:		std_logic;	-- 1 cycle delay for chip-enable
signal oe_i:			std_logic;	-- internal copy of RAM output-enable
signal oe_dly:		std_logic;	-- 1 cycle delay for output-enable
signal pixelRdy_i:		std_logic;	-- indicates pixel data is available for display
signal pixelRdy_dly:	std_logic_vector(2 downto 1);	-- 2 cycle delay for pixelRdy
signal blank_i:		std_logic;	-- indicates image blanking interval
signal blank_dly:	std_logic_vector(2 downto 1);	-- 2 cycle delay for blanking
signal rgb_i:			unsigned(5 downto 0);	-- internal copy of RGB signals
signal rgb_dly:		unsigned(5 downto 0);	-- 1 cycle delay of RGB
begin

vcc		<= HI;
we_n	<= HI;	-- disable all writing to image RAM

-- generate the horizontal sync
horizSync:
	sync 
	generic map(
		freq		=> freq,			-- master dot-clock frequency
		period	=> 32,				-- scanline period (32 us)
		start		=> 26,				-- start of hsync pulse
		width		=> 4					-- duration of hsync pulse
	) 
	port map(
		rst			=> rst,
		clk			=> clk,				-- master clock
		cke			=> vcc,				-- clock always enabled
		sync		=> hsync_i,		-- horizontal sync pulse
		gate		=> vertGate,	-- clock-enable for the vertical sync generator
		cnt			=> pixelCnt		-- index of current pixel in scanline (increments every clock)
	);		

-- generate the vertical sync
vertSync:
	sync
	generic map(
		freq		=> 31,				-- scanline frequency (32 us => 31 KHz)
		period	=> 16_784,		-- image frame period
		start		=> 15_700,		-- start of vsync pulse
		width		=> 64					-- duration of vsync pulse
	)
	port map(
		rst			=>rst,
		clk			=> clk,				-- master clock
		cke			=> vertGate,	-- clock vsync generator only once per scanline
		sync		=> vsync_i,		-- vertical sync pulse
		cnt			=> lineCnt,		-- index of current scanline in image frame
		gate		=> open
	);

-- compute the RAM address of the current pixel data
addrRAM:
	addrgen
	generic map(
		w						=> w,						-- image width in pixels
		h						=> h,						-- image height in scanlines
		datawidth		=> datawidth,		-- width of RAM data
		pixelWidth	=> pixBits			-- number of bits per pixel
	)
	port map(
		pixelCnt		=> pixelCnt,		-- current pixel index in scanline
		lineCnt			=> lineCnt,			-- current scanline in image frame
		offset			=> addrOffset,	-- address offset to image data
		ce_n				=> ce_i,				-- RAM chip-enable control
		oe_n				=> oe_i,				-- RAM output-enable control
		addr				=> addr_i,			-- RAM address for current pixel
		pixelRdy		=> pixelRdy_i,	-- indicates when pixel data is available
		blank				=> blank_i			-- indicates when image is blanked
	);

-- generate the RGB from the pixel value	
colorgen:
	colormap
	generic map(
		datawidth		=> datawidth,				-- width of RAM data
		pixelWidth	=> pixBits					-- bits per pixel
	)
	port map(
		rst					=> rst,
		clk					=> clk,
		pixdata			=> data,						-- RAM data containing pixel values
		load				=> pixelRdy_dly(2),	-- load data when pixels are available
		blank				=> blank_dly(2),		-- blanking signal
		rgb					=> rgb_i						-- RGB values for the current pixel
	);

-- delay the signals so they reach the right places at the right times
process(clk)
begin
	if clk'event and clk=HI then
		if rst=HI then	-- reset the delayed signals to the appropriate values
			hsync_dly			<= (others=>HI);	-- no horiz. sync pulse
			vsync_dly			<= (others=>HI);	-- no vert. sync pulse
			addr_dly			<= (others=>LO);	-- RAM address at zero
			ce_dly				<= HI;						-- disable RAM
			oe_dly				<= HI;						-- disable RAM outputs
			blank_dly			<= (others=>NO);	-- no blanking
			pixelRdy_dly	<= (others=>NO);	-- no pixel data available
			rgb_dly				<= (others=>LO);	-- RGB at black level
		else	-- delay the signals through flip-flops
			hsync_dly(1) <= hsync_i;				-- delay horizontal sync 3 cycles
			hsync_dly(2) <= hsync_dly(1);
			hsync_dly(3) <= hsync_dly(2);
			vsync_dly(1) <= vsync_i;				-- delay vertical sync 3 cycles
			vsync_dly(2) <= vsync_dly(1);
			vsync_dly(3) <= vsync_dly(2);
			addr_dly <= addr_i;							-- delay RAM address 1 cycle
			ce_dly <= ce_i;									-- delay RAM chip-enable 1 cycle
			oe_dly <= oe_i;									-- delay RAM output-enable 1 cycle
			blank_dly(1) <= blank_i;				-- delay blanking signal 2 cycles
			blank_dly(2) <= blank_dly(1);
			pixelRdy_dly(1) <= pixelRdy_i;	-- delay pixel data available signal 2 cycles
			pixelRdy_dly(2) <= pixelRdy_dly(1);
			rgb_dly <= rgb_i;								-- delay RGB signals 1 cycle
		end if;
	end if;
end process;

-- pass appropriately-delayed signals to the outputs
hsync_n <= hsync_dly(3);	-- output horiz. sync pulse
vsync_n <= vsync_dly(3);	-- output vert. sync pulse
blank_n <= not blank_dly(2);	-- output blanking signal
address <= addr_dly;			-- output RAM address
ce_n <= ce_dly;						-- output RAM chip-enable
oe_n <= oe_dly;						-- output RAM output-enable
rgb <= rgb_dly;						-- output RGB signals
	
end arch;
